'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var autocomplete = require('@codemirror/autocomplete');
var language = require('@codemirror/language');
var highlight = require('@lezer/highlight');

const FireStoreLanguage = language.StreamLanguage.define({
    name: "firestore",
    startState: (indentUnit) => {
        return {};
    },
    token: (stream, state = {}) => {
        if (stream.match("db")) {
            state.db = true;
            return "namespace";
        }
        if (stream.match("collection")) {
            if (state.db) {
                state.collection = true;
                return "function";
            }
        }
        if (stream.match("where")) {
            if (state.collection) {
                state.where = true;
                return "function";
            }
        }
        if (stream.match("limit")) {
            if (state.where) {
                state.limit = true;
                return "function";
            }
        }
        if (stream.match("get")) {
            if (state.limit) {
                state.limit = false;
                return "function";
            }
        }
        if (stream.match(/"(?:[^\\"]|\\.)*"/)) {
            if (state.collection) {
                return "string";
            }
            if (state.where) {
                state.where = false;
                state.whereValue = true;
                return "string";
            }
            if (state.whereValue) {
                state.whereValue = false;
                return "string";
            }
            if (stream.match("==")) {
                if (state.whereValue) {
                    state.whereValue = false;
                    state.whereOperator = true;
                    return "operator";
                }
            }
            if (stream.match(/[0-9]+/)) {
                if (state.limit) {
                    return "number";
                }
            }
        }
        stream.next();
        return null;
    },
    blankLine: (state, indentUnit) => { },
    copyState: (state) => { },
    indent: (state, textAfter, context) => {
        return 1;
    },
    languageData: {
        commentTokens: { line: ";" },
    },
    tokenTable: {
        db: highlight.tags.namespace,
        dot: highlight.tags.punctuation,
        collection: highlight.tags.function(highlight.tags.variableName),
        where: highlight.tags.function(highlight.tags.variableName),
        limit: highlight.tags.function(highlight.tags.variableName),
        get: highlight.tags.function(highlight.tags.atom),
        lParen: highlight.tags.punctuation,
        rParen: highlight.tags.punctuation,
        string: highlight.tags.string,
    },
});
const FireStoreCompletion = FireStoreLanguage.data.of({
    autocomplete: autocomplete.completeFromList([
        { label: "db", type: "namespace" },
        { label: "collection", type: "function" },
        { label: "where", type: "function" },
        { label: "limit", type: "function" },
        { label: "get", type: "function" },
    ]),
});
function firestore() {
    return new language.LanguageSupport(FireStoreLanguage, [FireStoreCompletion]);
}

exports.FireStoreCompletion = FireStoreCompletion;
exports.FireStoreLanguage = FireStoreLanguage;
exports.firestore = firestore;
